#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/************   The type OBJECT            ************/

#define FL_LABEL_SIZE		64

typedef struct objs {
   int objclass;        /* What type of object */
   int type;            /* The type within the class */
   int boxtype;         /* The type of the bounding box */
   float x,y,w,h;       /* The bounding box */
   int col1,col2;       /* Two possible colors */

   char label[FL_LABEL_SIZE];	/* The label */
   int align;		/* Label or text alignment */
   int lcol;		/* Color of the label */
   float lsize; 	/* Size of the label */
   int lstyle;		/* Style of the text label */

   int (*handle)(struct objs *, int, float, float, char);
			/* Handling procedure. */

   int *spec;           /* pointer to special stuff for object */

   int pushed;		/* wheter pushed */
   int focus; 		/* wheter focussed */
   int belowmouse;	/* Whether currently below the mouse */

   int frozen;		/* Whether it will be redrawn */
   int active;		/* Whether active */
   int input;		/* Whether receiving input */
   int visible;         /* Whether being displayed */
   int radio;           /* Whether a radio object */
   int automatic;       /* Whether this object gets timer events */
   void (*object_call_back)(struct objs *, long);
			/* The call-back routine */
   long argument;	/* Its argument */

   struct objs *next;	/* Next object in the form */
   struct objs *prev;	/* Previous object in the form */
   struct objs *form;	/* Form to which object belong */
} FL_OBJECT;

/************   General Constants          ************/

#ifndef NULL
#define NULL                    0
#endif

/* Conversion program to turn version 1.5 Form Designer files 
   int 2.0 files.
*/

#define NL	10

char tmpstr[200];

char *get_label(FL_OBJECT *obj)
{
  int i=0, j=0;
  do
    if (obj->label[i] == NL) {tmpstr[j++] = '\\'; tmpstr[j++] = 'n';}
    else tmpstr[j++] = obj->label[i];
  while (obj->label[i++] != NULL);
  return tmpstr;
}

/********************* Actual Stuff **********************/

char main_name[64];
FILE *fin,*fout;
char fname[40],name[40], cbname[40], argname[40];

void save_object(FL_OBJECT *obj)
/* Writes an obj description to a file. */
{
  fprintf(fout,"\n--------------------\n");
  fprintf(fout,"class: %d\n",obj->objclass);
  fprintf(fout,"type: %d\n",obj->type);
  fprintf(fout,"box: %f %f %f %f\n",obj->x,obj->y,obj->w,obj->h);
  fprintf(fout,"boxtype: %d\n",obj->boxtype);
  fprintf(fout,"colors: %d %d\n",obj->col1,obj->col2);
  fprintf(fout,"alignment: %d\n",obj->align);
  fprintf(fout,"style: %d\n",obj->lstyle);
  fprintf(fout,"size: %f\n",obj->lsize);
  fprintf(fout,"lcol: %d\n",obj->lcol);
  fprintf(fout,"label: %s\n",get_label(obj));
  fprintf(fout,"name: %s\n",name);
  fprintf(fout,"callback: %s\n",cbname);
  fprintf(fout,"argument: %s\n",argname);
}

void convert_form()
/* Converts one form definition */
{
  int i,onumb;
  FL_OBJECT *obj;
  float w,h;
  obj = (FL_OBJECT *) malloc(sizeof(FL_OBJECT));
  /* read the form name */
  fread(fname,sizeof(char),40,fin);
  fread(&w,sizeof(float),1,fin);
  fread(&h,sizeof(float),1,fin);
  fprintf(fout,"\n=============== FORM ===============\n");
  fprintf(fout,"Name: %s\n",fname);
  fprintf(fout,"Width: %f\n",w);
  fprintf(fout,"Height: %f\n",h);
  /* read the object numb */
  fread(&onumb,sizeof(int),1,fin);
  fprintf(fout,"Number of Objects: %d\n",onumb);
  for (i = 0; i<onumb; i++)
  {
    fread(name,sizeof(char),40,fin);
    fread(cbname,sizeof(char),40,fin);
    fread(argname,sizeof(char),40,fin);
    fread(obj,sizeof(FL_OBJECT),1,fin);
    save_object(obj);
  }
}

/************/

void convert_file(char oldname[], char newname[])
/* Converts old form definition to new one. */
{
  int i;
  /* Open the files */
  fin = fopen(oldname,"r");
  if (fin == (FILE *) NULL)
    {fprintf(stderr,"Cannot open file %s\n",oldname); return;}
  fout = fopen(newname,"w");
  if (fout == (FILE *) NULL)
    {fprintf(stderr,"Cannot create file %s\n",newname); return;}
  /* Create the header stuff */
  fread(&i,sizeof(int),1,fin);
  fprintf(fout,"Magic: 12321\n\n");
  fprintf(fout,"Internal Form Definition File\n");
  fprintf(fout,"    (do not change)\n\n");
  fprintf(fout,"Number of forms: %d\n",i);
  while (i>0) {convert_form();i--;}
  if (fread(main_name,64,1,fin) <= 0) strcpy(main_name,"create_the_forms");
  fprintf(fout,"\n==============================\n%s\n",main_name);
  fclose(fin);
  fclose(fout);
}

main(int argc, char *argv[])
{
  char inname[256], outname[256];
  if (argc != 2)
    { fprintf(stderr,"Usage: %s <filename>\n",argv[0]); exit(1); }
  strcpy(inname,argv[1]);
  strcat(inname,".def");
  strcpy(outname,argv[1]);
  strcat(outname,".fd");
  convert_file(inname,outname);
}
