/*
 * fd_groups.c
 *
 * This file is part of the Form Designer.
 *
 * It maintains a list of groups in the form. Each group can be
 * assigned a name.
 *
 * Written by Mark Overmars
 *
 * Version 2.2a
 * Date  Jun 18, 1993
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "forms.h"
#include "fd_main.h"
#include "fd_theforms.h"

/**** Data Structure maintaining groups in current form ****/

static FL_OBJECT *begobj[MAXGROUP];	/* Begin objects of the groups */
static FL_OBJECT *endobj[MAXGROUP];	/* End objects of the groups */

/**** Call-back routines ****/

void group_cb(FL_OBJECT *obj, long arg)
/* Sets the current group in the form */
{
  int nn = fl_get_browser(groupbrowser);
  if (nn>0)
    addgroupto_selection(begobj[nn]);
  else if (nn<0)
    deletegroupfrom_selection(begobj[-nn]);
  redraw_the_form(0);
}

void changegroupname_cb(FL_OBJECT *obj, long arg)
/* Changes the name of the current group */
{
  char name[40], cbname[40],argname[40];
  int i,numb = 0;
  if (cur_form == NULL) return;
  for (i=1; i<=fl_get_browser_maxline(groupbrowser); i++)
    if (fl_isselected_browser_line(groupbrowser,i))
    {
      if (numb != 0)
	{fl_show_message("Cannot change name of multiple groups","","");return;}
      numb = i;
    }
  if (numb == 0)
    {fl_show_message("No group was selected.","","");return;}
  get_object_name(begobj[numb],name,cbname,argname);
  strcpy(name,fl_show_input("Give the new group name",name));
  set_object_name(begobj[numb],name,cbname,argname);
  fillin_groups();
  changed = 1;
  return;
}
  
/************ MAIN ROUTINES ******************/

void fillin_groups()
/* fills in the current list of groups. */
{
  char name[40], cbname[40],argname[40];
  FL_OBJECT *obj, *obj2;
  int i = 1;
  if (cur_form==NULL) {fl_clear_browser(groupbrowser); return;}
  fl_freeze_form(controlform);
  fl_clear_browser(groupbrowser);
  /* Check whether groups are empty. */
  obj = cur_form->first;
  while (obj != NULL)
  {
    obj2 = obj;
    obj = obj->next;
    if (obj2->objclass == FL_END_GROUP &&
       (obj2->prev != NULL && obj2->prev->objclass == FL_BEGIN_GROUP))
    {
      fl_delete_object(obj2->prev);
      fl_delete_object(obj2);
    }
  }
  /* Put the still existing groups in the list */
  obj = cur_form->first;
  while (obj != NULL)
  {
    if (obj->objclass == FL_BEGIN_GROUP)
    {
      begobj[i] = obj;
      get_object_name(obj,name,cbname,argname);
      if (name[0] == '\0') strcpy(name,"<no name>");
      fl_add_browser_line(groupbrowser,name);
      if (is_selected(obj)) fl_select_browser_line(groupbrowser,i);
    }
    else if (obj->objclass == FL_END_GROUP)
    {
      endobj[i++] = obj;
    }
    obj = obj->next;
  }
  fl_unfreeze_form(controlform);
}
