/*
 * fd_help.c
 * 
 * This file is part of the Form Designer.
 *
 * It contains the routines for the help feature of the designer.
 *
 * Written by Mark Overmars
 *
 * Version 2.2a
 * Date  Jun 18, 1993
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "forms.h"
#include "fd_main.h"
#include "fd_theforms.h"

char mainhelp[][80] = {
    "@C4@l@b@cForm Designer",
    "  ",
    "@C4@b@cVersion 2.2",
    "  ",
    "@C4@s@c@iWritten by Mark Overmars",
    "  ",
    "The Form Designer is a program for interactively designing dialogue",
    "forms for use with the Forms Library. The prgram is extremely simple",
    "to use. Press the different buttons to get help on the different topics.",
    "  ",
    "@bNotice:",
    "Since version 2.0 the Form Designer uses a different file format",
    "than previous versions. Use the program conv15to20 to convert your old",
    "files to the new format",
    ""};

char generalhelp[][80] = {
    "@l@b@cGeneral Information",
    "  ",
    "@bAdding forms",
    "To add a form, press the button labeled 'Add Form'. A red outline will",
    "appear in the main window. Use your mouse to indicate the size of the",
    "form. Next you are asked for the name of the form.",
    " ",
    "@bChanging the form size",
    "It is possible to change the size of a form later. To this end select",
    "the bottom face, using the right mouse button. Next grab the right-top",
    "corner, using the middle mouse button, and scale it to its new size.",
    " ",
    "@bDeleting a form",
    "Use the button labeled 'Delete Form' to delete the current form. You",
    "will have to acknowledge the deletion.",
    " ",
    "@bChanging the name of the form",
    "This can be achieved by pressing the button labeled 'Form Name' below",
    "the list of forms.",
    " ",
    "@bAligning objects",
    "When pressing this button a form appears at the top right of the screen.", 
    "Now make a selection of the objects and press one of the buttons in the", 
    "form to align them at the left, horizontal centers, right, etc. It is", 
    "also possible to put them with equal gaps between them. In this form you", 
    "can also indicating the snapping value which indicates the number of", 
    "pixels to which values are rounded when putting objects in your form.", 
    ""};

char mousehelp[][80] = {
    "@l@b@cUsing the Mouse",
    "  ",
    "The mouse is used to add objects, select objects and move them around",
    "or scale them. Sometimes, holding down the <Shift> key influences the",
    "action taken.",
    " ",
    "@bLeft Mouse Button",
    "The left mouse button is used to add objects to a form. To avoid",
    "mistakes, the <Shift> key must be held down while adding an object. ",
    " ",
    "@bMiddle Mouse Button",
    "The middle mouse button is used to move or scale selected objects.",
    "When you press the mouse inside the red outline you can move the",
    "selected objects around. When you press it in one of the four red",
    "squares you can scale the object. Holding down the <Shift> key while",
    "moving an object results in a copy being made which can then be moved.",
    " ",
    "@bRight Mouse Button",
    "The right mouse button is used for making selections. You can either",
    "press the mouse button inside an object to select it, or draw an",
    "outline around all objects you want to select. By holding down the",
    "<Shift> key you can add or remove objects from the current selection.",
    ""};

char keyhelp[][80] = {
    "@l@b@cFunction Keys",
    "  ",
    "The function keys are used to perform certain actions on the current",
    "selection. You can either use the function keys on the keyboard or",
    "the corresponding buttons on the screen.",
    " ",
    "@bF1  Attributes",
    "Change the attributes of the current selection. This only works if the",
    "objects in the selection are all of the same class. When only one object",
    "is selected, also label and name can be changed.",
    " ",
    "@bF2  Lower selection",
    "Lowers the objects in the current selection, i.e., places them below all",
    "other objects on the form.",
    " ",
    "@bF3  Raise selection",
    "Raises all objects in the selection, i.e., places them on top of all other",
    "objects in the form.",
    " ",
    "@bF4  Select all",
    "Puts all objects in a selection.",
    " ",
    "@bF5  Show selection",
    "Makes the all objects in the current selection visible. You can first",
    "use F4 to select all objects.",
    " ",
    "@bF6  Hide selection",
    "Hides all objects in the current selection. Use F5 to make them",
    "visible again.",
    " ",
    "@bF7  Group selection",
    "Places all objects in the selection in a group. If the objects are",
    "already part of a group, they are removed from the old group. Groups",
    "that become empty this way are discarded. You will be asked to provide a",
    "name for the group. This will be the C-name under which the group is",
    "known to the application program.",
    " ",
    "@bF8  Ungroup selection",
    "Takes all objects in the current selection out of their groups. Empty",
    "groups will be discarded.",
    " ",
    "@bF9  Copy selection to buffer",
    "Copies the objects in the current selection to the buffer. They can",
    "later be added to the same or another form using the operation paste.",
    " ",
    "@bF10 Paste buffer into form",
    "Pastes the objects in the buffer (placed there by the last copy or cut",
    "operation) into the form. You can indicate the required position. The",
    "pasted objects will form the new selection.",
    " ",
    "@bF11",
    "Not used.",
    " ",
    "@bF12 Cut selection into the buffer",
    "Deletes the current selection. The deleted objects will be placed in",
    "the buffer and can be pasted back into the form (or another form) later.",
    ""};

char grouphelp[][80] = {
    "@l@b@cGroups",
    "  ",
    "The Form Designer has a simple mechanism for dealing with groups inside",
    "forms. To create a group of objects, select all of them, using the right",
    "mouse button, and press key <F7>. The group name you indicate is added",
    "to the list of groups. Whenever you select all elements in a group the",
    "group name is shown in red. You can also select or deselect all elements",
    "in a group by pressing the mouse on the group name. Groups whose elements",
    "get deleted automatically disappear.",
    " ",
    "@bAdding an object to a group",
    "It is not possible to directly add an object to an existing group. You",
    "should first select the group and the new object and next press <F7> to",
    "create a new group. The old group will be removed and a new group,",
    "including the new object, will be created. You will have to type in the",
    "group name again.",
    " ",
    "@bChanging the name of a group",
    "To change the name of a group, select it (and deselect the others) and",
    "press the button labeled 'Group Name' below the list of groups.",
    ""};

char savehelp[][80] = {
    "@l@b@cSaving and Loading",
    "  ",
    "To save the current set of forms, press the button labeled 'Save Forms'.",
    "The file selector will appear in which you can indicate the name under",
    "which to save the form. This name should end with .fd. Three files will",
    "be generated: <name>.c, <name>.h and <name>.fd. See the documentation",
    "for their purpose.",
    " ",
    "A new set of forms can be loaded using the button labeled 'Load Forms'.",
    "It is also possible to merge the new forms with the current set using",
    "the button labeled 'Merge Forms'.",
    "  ",
    "@bNotice:",
    "Since version 2.0 the Form Designer uses a different file format",
    "than previous versions. Use the program conv15to20 to convert your old",
    "files to the new format",
    ""};

char testhelp[][80] = {
    "@l@b@cTesting Forms",
    "  ",
    "To test the current form press the button labeled 'Test Form'. The form",
    "will be centered on the screen and in the top right corner a new form",
    "will appear. Now you can play with the form. In the browser the objects",
    "are shown that will be returned to the application program or the callback",
    "routines that will be called. In this way you can verify whether each",
    "object has got a name or callback routine associated with it. To stop",
    "testing press the button labeled 'Stop Testing'.",
    " ",
    "Note that changes you make to the objects, e.g. filling in an input field,",
    "will not be saved when saving the forms.",
    ""};

static void load_it(char str[][80])
/* Loads the help browser with specific information. */
{
  int i;
  fl_freeze_form(helpform);
  fl_clear_browser(helpbrowser);
  for (i=0; str[i][0] != '\0'; i++)
    fl_add_browser_line(helpbrowser,str[i]);
  fl_unfreeze_form(helpform);
}

void help_cb(FL_OBJECT *obj, long arg)
/* Shows the help form. */
   {fl_show_form(helpform,FL_PLACE_POSITION,FALSE,NULL);}

void exithelp_cb(FL_OBJECT *obj, long arg)
/* Stop showing the help window */
  { fl_hide_form(helpform); }

void showhelp_cb(FL_OBJECT *obj, long arg)
/* Shows a particular help item. */
{
  switch (arg) {
    case 1: load_it(mainhelp); break;
    case 2: load_it(generalhelp); break;
    case 3: load_it(mousehelp); break;
    case 4: load_it(keyhelp); break;
    case 5: load_it(grouphelp); break;
    case 6: load_it(testhelp); break;
    case 7: load_it(savehelp); break;
  }
}
